%Steady State Transport Model for Concentration Profile is a Sediment Cap
%Developed by Danny Reible and David Lampert

%This model assumes advection, diffusion, dispersion, and first-order decay
%with boundary layer mass transfer to compute the average bioturbation
%layer concentration in a sediment cap at steady state.

n=1000                      %Number of simulations

%Input Parameter Arithmetic Averages and Standard Deviations

Dwavg=6e-6;                  %cm2/s
Dwstdev=1e-6;                %cm2/s
Uavg=100;                    %cm/yr
Ustdev=10;                   %cm/yr
hcapavg=60;                  %cm
hcapstdev=5;                 %cm
hbioavg=10;                  %cm
hbiostdev=5;                 %cm
lambda1avg=1e-8;             %s-1
lambda1stdev=1e-8;           %s-1
lambda2avg=1e-8;             %s-1
lambda2stdev=1e-8;           %s-1
porosityavg=0.55;            %-
porositystdev=0.1;           %-
particledensityavg=2.6;      %g/cm3
particledensitystdev=0.15;   %g/cm3
focavg=0.05;                 %-
focstdev=0.005;              %-
logKocavg=4.3;               %log(L/kg)
logKocstdev=0.1;             %log(L/kg)
Dbiopwavg=100;               %cm2/yr
Dbiopwstdev=100;             %cm2/yr
Dbiopavg=5;                  %cm2/yr
Dbiopstdev=5;                %cm2/yr
kblavg=3;                    %cm/hr
kblstdev=1;                  %cm/hr

%Input Parameter Correlations

correl=speye(13);
correl(4,11)=0.7;            %Correlation Coeff. for hbio & Dbiopw
correl(11,4)=correl(4,11);
correl(4,12)=0.7;            %Correlation Coeff. for hbio & Dbiop
correl(12,4)=correl(4,12);
correl(5,10)=-0.5;           %Correlation Coeff. for lambda & logKoc
correl(10,5)=correl(5,10);
correl(11,12)=0.8;           %Correlation Coeff. for Dbiop & Dbiopw
correl(12,11)=correl(11,12);

%"Input" is the 13x2 matrix of averages and standard deviations

Input(1,1)=Dwavg;
Input(1,2)=Dwstdev;
Input(2,1)=Uavg;
Input(2,2)=Ustdev;
Input(3,1)=hcapavg;
Input(3,2)=hcapstdev;
Input(4,1)=hbioavg;
Input(4,2)=hbiostdev;
Input(5,1)=lambda1avg;
Input(5,2)=lambda1stdev;
Input(6,1)=lambda2avg;
Input(6,2)=lambda2stdev;
Input(7,1)=porosityavg;
Input(7,2)=porositystdev;
Input(8,1)=particledensityavg;
Input(8,2)=particledensitystdev;
Input(9,1)=focavg;
Input(9,2)=focstdev;
Input(10,1)=logKocavg;
Input(10,2)=logKocstdev;
Input(11,1)=Dbiopwavg;
Input(11,2)=Dbiopwstdev;
Input(12,1)=Dbiopavg;
Input(12,2)=Dbiopstdev;
Input(13,1)=kblavg;
Input(13,2)=kblstdev;

%Create the transformation matrix "cholesky."  "covariance" is an
%intermediate covariance matrix.

for i=1:13
    for j=1:13
        covariance(i,j)=correl(i,j)*Input(i,2)*Input(j,2);
    end
end

cholesky=transpose(chol(covariance));

%Calculate the distributions. "Norm," "Rand," and "CDF" are intermediate 
%matrices of parameters for the correlations.

Norm(1:13,1:n)=normrnd(0,1,13,n);
for i=1:n
    Rand(1:13,i)=cholesky*Norm(1:13,i)+Input(1:13,1);
end
for i=1:n
    CDF(1:13,i)=normcdf(Rand(1:13,i),Input(1:13,1),Input(1:13,2));
end

Dw=norminv(CDF(1,1:n),Dwavg,Dwstdev);
U=logninv(CDF(2,1:n),log(Uavg/sqrt(1+(Ustdev/Uavg)^2)),sqrt(log(1+...
    (Ustdev/Uavg)^2)));
hcap=norminv(CDF(3,1:n),hcapavg,hcapstdev);
hbio=logninv(CDF(4,1:n),log(hbioavg/sqrt(1+(hbiostdev/hbioavg)^...
    2)),sqrt(log(1+(hbiostdev/hbioavg)^2)));
lambda1=logninv(CDF(5,1:n),log(lambda1avg/sqrt(1+(lambda1stdev/...
    lambda1avg)^2)),sqrt(log(1+(lambda1stdev/lambda1avg)^2)));
lambda2=logninv(CDF(6,1:n),log(lambda2avg/sqrt(1+(lambda2stdev/...
    lambda2avg)^2)),sqrt(log(1+(lambda2stdev/lambda2avg)^2)));
porosity=norminv(CDF(7,1:n),porosityavg,porositystdev);
particledensity=norminv(CDF(8,1:n),particledensityavg,...
    particledensitystdev,n,1);
foc=logninv(CDF(9,1:n),log(focavg/sqrt(1+(focstdev/focavg)^...
    2)),sqrt(log(1+(focstdev/focavg)^2)));
logKoc=norminv(CDF(10,1:n),logKocavg,logKocstdev);
Dbiopw=logninv(CDF(11,1:n),log(Dbiopwavg/sqrt(1+(Dbiopwstdev/...
    Dbiopwavg)^2)),sqrt(log(1+(Dbiopwstdev/Dbiopwavg)^2)));
Dbiop=logninv(CDF(12,1:n),log(Dbiopavg/sqrt(1+(Dbiopstdev/Dbiopavg)^...
    2)),sqrt(log(1+(Dbiopstdev/Dbiopavg)^2)));
kbl=logninv(CDF(13,1:n),log(kblavg/sqrt(1+(kblstdev/kblavg)^...
    2)),sqrt(log(1+(kblstdev/kblavg)^2))).*24.*365;

%These are intermediate calculations for the solution

D1=porosity.^(4/3).*Dw*365*86400+1.69.*(hcap./100).^1.53.*U;
Rf=1+(1-porosity)./porosity.*10.^logKoc.*foc.*particledensity;
D2=D1+Dbiopw+Dbiop.*Rf.*porosity;
heff=hcap-hbio;
Pe1=U.*heff./D1;
Da1=porosity.*lambda1.*heff.^2./D1*365*86400;
beta=(Pe1.^2./4+Da1).^0.5;
Pe2=U.*hbio./D2;
Da2=porosity.*lambda2.*hbio.^2./D2*365*86400;
gamma=(Pe2.^2./4+Da2).^0.5;

%And the values for Cbio, Cbl, and Cbioavg

Cbio=beta.*exp(Pe1./2).*D1./heff./sinh(beta)./(Pe1.*D1./2./heff+beta.*...
    D1.*cosh(beta)./heff./sinh(beta)+gamma.*D2.*cosh(gamma)./hbio./...
    sinh(gamma)-Pe2.*D2./2./hbio-2.*gamma.^2.*D2.^2./(2*hbio.^2.*kbl.*...
    (sinh(gamma)).^2+gamma.*hbio.*D2.*sinh(2.*gamma)));

Cbl=exp((Pe1+Pe2)./2)./(kbl.*hbio./D2+Pe2./2+gamma.*cosh(gamma)./...
    sinh(gamma))./(Pe1.*sinh(beta).*sinh(gamma)./2./beta./gamma+...
    cosh(beta).*sinh(gamma)./gamma+heff.*D2.*sinh(beta).*cosh(gamma)./...
    beta./hbio./D1-Pe2.*heff.*D2.*sinh(beta).*sinh(gamma)./2./beta./...
    gamma./hbio./D1-2.*gamma.*heff.*D2.^2.*sinh(beta)./(hbio.*D1.*...
    beta)./(Pe2.*D2.*sinh(gamma)+2.*hbio.*kbl.*sinh(gamma)+2.*gamma.*...
    D2.*cosh(gamma)));

Cbioavg=((Cbl.*exp(-Pe2./2)-Cbio.*exp(-gamma))./2./sinh(gamma))./...
    (Pe2./2+gamma).*(exp(Pe2./2+gamma)-1)+((Cbio.*exp(gamma)-Cbl.*...
    exp(-Pe2./2)))./2./sinh(gamma)./(Pe2./2-gamma).*(exp(Pe2/2-gamma)-1);

%Now analyze the data

Cbioavg=sort(Cbioavg);
p=tiedrank(Cbioavg)./n;
Cbioavgavg=mean(Cbioavg)
Cbioavgstd=std(Cbioavg)

plot(Cbioavg,p)
axis([0 1 0 1])
xlabel('Average Bioturbation Layer Concentration')
ylabel('P(x<Average Bioturbation Layer Concentration)')
title('Sediment Cap Design Model Cumulative Distribution Function')
